# Generated by roxygen2: do not edit by hand

export(AIFEBaseModel)
export(AIFEMaster)
export(BaseModelAlbert)
export(BaseModelBert)
export(BaseModelCore)
export(BaseModelDebertaV2)
export(BaseModelDistilBERT)
export(BaseModelEuroBert)
export(BaseModelFunnel)
export(BaseModelMPNet)
export(BaseModelModernBert)
export(BaseModelRoberta)
export(BaseModelRobertaXML)
export(BaseModelXmod)
export(BaseModelsIndex)
export(ClassifiersBasedOnTextEmbeddings)
export(DataManagerClassifier)
export(DataSetsIndex)
export(EmbeddedText)
export(HuggingFaceTokenizer)
export(LargeDataSetBase)
export(LargeDataSetForText)
export(LargeDataSetForTextEmbeddings)
export(ModelsBasedOnTextEmbeddings)
export(TEClassifierParallel)
export(TEClassifierParallelPrototype)
export(TEClassifierProtoNet)
export(TEClassifierRegular)
export(TEClassifierSequential)
export(TEClassifierSequentialPrototype)
export(TEClassifiersBasedOnProtoNet)
export(TEClassifiersBasedOnRegular)
export(TEFeatureExtractor)
export(TextEmbeddingModel)
export(TokenizerBase)
export(TokenizerIndex)
export(WordPieceTokenizer)
export(add_missing_args)
export(auto_n_cores)
export(build_documentation_for_model)
export(build_layer_overview_base_models)
export(build_layer_stack_documentation_for_vignette)
export(calc_standard_classification_measures)
export(calc_tokenizer_statistics)
export(cat_message)
export(check_aif_py_modules)
export(check_all_args)
export(check_class_and_type)
export(class_vector_to_py_dataset)
export(clean_pytorch_log_transformers)
export(cohens_kappa)
export(create_dir)
export(create_object)
export(create_synthetic_units_from_matrix)
export(data.frame_to_py_dataset)
export(extract_column_from_py_dataset)
export(fleiss_kappa)
export(generate_args_for_tests)
export(generate_embeddings)
export(generate_id)
export(generate_tensors)
export(get_TEClassifiers_class_names)
export(get_alpha_3_codes)
export(get_batches_index)
export(get_called_args)
export(get_coder_metrics)
export(get_current_args_for_print)
export(get_depr_obj_names)
export(get_desc_for_core_model_architecture)
export(get_file_extension)
export(get_fixed_test_tensor)
export(get_layer_documentation)
export(get_n_chunks)
export(get_param_def)
export(get_param_dict)
export(get_param_doc_desc)
export(get_parameter_documentation)
export(get_py_package_version)
export(get_py_package_versions)
export(get_recommended_py_versions)
export(get_synthetic_cases_from_matrix)
export(get_test_data_for_classifiers)
export(get_time_stamp)
export(gwet_ac)
export(inspect_tmp_dir)
export(install_aifeducation)
export(install_aifeducation_studio)
export(install_protocol_buffer_compiler)
export(install_py_modules)
export(kendalls_w)
export(knnor)
export(knnor_is_same_class)
export(kripp_alpha)
export(load_all_py_scripts)
export(load_from_disk)
export(load_py_scripts)
export(long_load_target_data)
export(matrix_to_array_c)
export(monitor_test_time_on_CI)
export(output_message)
export(prepare_r_array_for_dataset)
export(prepare_session)
export(print_message)
export(py_dataset_to_embeddings)
export(random_bool_on_CI)
export(read_log)
export(read_loss_log)
export(reduce_to_unique)
export(reset_log)
export(reset_loss_log)
export(run_py_file)
export(save_to_disk)
export(set_codecarbon_logger)
export(set_transformers_logger)
export(start_aifeducation_studio)
export(summarize_args_for_long_task)
export(tensor_list_to_numpy)
export(tensor_to_matrix_c)
export(tensor_to_numpy)
export(to_categorical_c)
export(update_aifeducation)
export(write_log)
import(Rcpp)
import(doParallel)
import(foreach)
import(reshape2)
import(reticulate)
import(stats)
importFrom(methods,is)
importFrom(parallel,detectCores)
importFrom(reticulate,conda_create)
importFrom(reticulate,conda_remove)
importFrom(reticulate,condaenv_exists)
importFrom(reticulate,import)
importFrom(reticulate,import_builtins)
importFrom(reticulate,install_miniconda)
importFrom(reticulate,install_python)
importFrom(reticulate,py_config)
importFrom(reticulate,py_install)
importFrom(reticulate,py_module_available)
importFrom(reticulate,py_run_file)
importFrom(reticulate,virtualenv_exists)
importFrom(rlang,.data)
importFrom(rlang,caller_fn)
importFrom(rlang,fn_fmls)
importFrom(stringi,stri_replace_all)
importFrom(stringi,stri_split_fixed)
importFrom(stringi,stri_trans_tolower)
importFrom(utils,compareVersion)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,packageVersion)
importFrom(utils,person)
importFrom(utils,read.csv)
importFrom(utils,write.csv)
useDynLib(aifeducation, .registration = TRUE)
