% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_log.R
\name{output_message}
\alias{output_message}
\title{Print message}
\usage{
output_message(msg, trace, msg_fun)
}
\arguments{
\item{msg}{\code{string} Message that should be printed.}

\item{trace}{\code{bool} Silent printing (\code{FALSE}) or not (\code{TRUE}).}

\item{msg_fun}{\code{bool} value that determines what function should be used. \code{TRUE} for \code{message()}, \code{FALSE} for
\code{cat()}.}
}
\value{
This function returns nothing.
}
\description{
Prints a message \code{msg} if \code{trace} parameter is \code{TRUE} with current date with \code{message()} or \code{cat()}
function.
}
\seealso{
Other Utils Log Developers: 
\code{\link{cat_message}()},
\code{\link{clean_pytorch_log_transformers}()},
\code{\link{print_message}()},
\code{\link{read_log}()},
\code{\link{read_loss_log}()},
\code{\link{reset_log}()},
\code{\link{reset_loss_log}()},
\code{\link{write_log}()}
}
\concept{Utils Log Developers}
