% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_DocumentationDesc.R
\name{get_parameter_documentation}
\alias{get_parameter_documentation}
\title{Generate layer documentation}
\usage{
get_parameter_documentation(
  param_name,
  param_dict,
  as_list = TRUE,
  inc_param_name = TRUE
)
}
\arguments{
\item{param_name}{\code{string} Name of the parameter.}

\item{param_dict}{\code{list} storing the parameter description.}

\item{as_list}{\code{bool} If \code{TRUE} returns the element as part of a list.}

\item{inc_param_name}{\code{bool} If \code{TRUE} the documentation includes the name of the parameter.}
}
\value{
Returns a \code{string} containing the description written in rmarkdown.
}
\description{
Function for generating the documentation of a specific layer.
}
\seealso{
Other Utils Documentation: 
\code{\link{build_documentation_for_model}()},
\code{\link{build_layer_stack_documentation_for_vignette}()},
\code{\link{get_desc_for_core_model_architecture}()},
\code{\link{get_dict_cls_type}()},
\code{\link{get_dict_core_models}()},
\code{\link{get_dict_input_types}()},
\code{\link{get_layer_dict}()},
\code{\link{get_layer_documentation}()}
}
\concept{Utils Documentation}
