% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_ParameterDefinitions.R
\name{get_called_args}
\alias{get_called_args}
\title{Called arguments}
\usage{
get_called_args(n = 1L)
}
\arguments{
\item{n}{\code{int} level of the nested environments where to extract the arguments.}
}
\value{
Returns a named \code{list} of all arguments and their values.
}
\description{
Function for receiving all arguments that were called by a method or function.
}
\seealso{
Other Parameter Dictionary: 
\code{\link{BaseModelsIndex}},
\code{\link{DataSetsIndex}},
\code{\link{TokenizerIndex}},
\code{\link{doc_formula}()},
\code{\link{get_TEClassifiers_class_names}()},
\code{\link{get_depr_obj_names}()},
\code{\link{get_magnitude_values}()},
\code{\link{get_param_def}()},
\code{\link{get_param_dict}()},
\code{\link{get_param_doc_desc}()}
}
\concept{Parameter Dictionary}
