% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_Object_Management.R
\name{create_object}
\alias{create_object}
\title{Create object#'}
\usage{
create_object(class)
}
\arguments{
\item{class}{\code{string} Name of the class to be created.}
}
\value{
Returns an object of the requested class.
}
\description{
Support function for creating objects.
}
\seealso{
Other Utils Developers: 
\code{\link{auto_n_cores}()},
\code{\link{create_synthetic_units_from_matrix}()},
\code{\link{generate_id}()},
\code{\link{get_n_chunks}()},
\code{\link{get_synthetic_cases_from_matrix}()},
\code{\link{get_time_stamp}()},
\code{\link{matrix_to_array_c}()},
\code{\link{tensor_to_matrix_c}()},
\code{\link{to_categorical_c}()}
}
\concept{Utils Developers}
