% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python.R
\name{load_all_py_scripts}
\alias{load_all_py_scripts}
\title{Load and re-load all python scripts}
\usage{
load_all_py_scripts()
}
\value{
Function does nothing return. It loads the requested scripts.
}
\description{
Function loads or re-loads all python scripts within the package 'aifeducation'.
}
\seealso{
Other Utils Python Developers: 
\code{\link{get_py_package_version}()},
\code{\link{get_py_package_versions}()},
\code{\link{load_py_scripts}()},
\code{\link{run_py_file}()}
}
\concept{Utils Python Developers}
