% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_Studio.R
\name{create_data_embeddings_description}
\alias{create_data_embeddings_description}
\title{Generate description for text embeddings}
\usage{
create_data_embeddings_description(embeddings)
}
\arguments{
\item{embeddings}{Object of class \link{LargeDataSetForTextEmbeddings} or \link{EmbeddedText}.}
}
\value{
Returns a \code{shiny::tagList} containing the html elements for the user interface.
}
\description{
Function generates a description for the underling \link{TextEmbeddingModel} of
give text embeddings.
}
\seealso{
Other Utils Studio Developers: 
\code{\link{add_missing_args}()},
\code{\link{long_load_target_data}()},
\code{\link{summarize_args_for_long_task}()}
}
\concept{Utils Studio Developers}
\keyword{internal}
