% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python.R
\name{get_py_package_versions}
\alias{get_py_package_versions}
\title{Get versions of python components}
\usage{
get_py_package_versions()
}
\value{
Returns a list that contains the version number of python and
the versions of critical python packages. If a package is not available
version is set to \code{NA}.
}
\description{
Function for requesting a summary of the versions of all
critical python components.
}
\seealso{
Other Utils Python Developers: 
\code{\link{get_py_package_version}()},
\code{\link{load_all_py_scripts}()},
\code{\link{load_py_scripts}()},
\code{\link{run_py_file}()}
}
\concept{Utils Python Developers}
