% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_Studio.R
\name{long_load_target_data}
\alias{long_load_target_data}
\title{Load target data for long running tasks}
\usage{
long_load_target_data(file_path, selectet_column)
}
\arguments{
\item{file_path}{\code{string} Path to the file storing the target data.}

\item{selectet_column}{\code{string} Name of the column containing the target data.}
}
\value{
Returns a named factor containing the target data.
}
\description{
Function loads the target data for a long running task.
}
\details{
This function assumes that the target data is stored as a columns with the cases in the rows and the
categories in the columns. The ids of the cases must be stored in a column called "id".
}
\seealso{
Other Utils Studio Developers: 
\code{\link{add_missing_args}()},
\code{\link{create_data_embeddings_description}()},
\code{\link{summarize_args_for_long_task}()}
}
\concept{Utils Studio Developers}
