% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{auto_n_cores}
\alias{auto_n_cores}
\title{Number of cores for multiple tasks}
\usage{
auto_n_cores()
}
\value{
Returns \code{int} as the number of cores.
}
\description{
Function for getting the number of cores that should be used
for parallel processing of tasks. The number of cores is set to 75 \% of the
available cores. If the environment variable \code{CI} is set to \code{"true"} or if the
process is running on cran \code{2} is returned.
}
\seealso{
Other Utils Developers: 
\code{\link{create_object}()},
\code{\link{create_synthetic_units_from_matrix}()},
\code{\link{generate_id}()},
\code{\link{get_n_chunks}()},
\code{\link{get_synthetic_cases_from_matrix}()},
\code{\link{get_time_stamp}()},
\code{\link{matrix_to_array_c}()},
\code{\link{tensor_to_matrix_c}()},
\code{\link{to_categorical_c}()}
}
\concept{Utils Developers}
