% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python_data_management.R
\name{create_py_dataset_cache_file_path}
\alias{create_py_dataset_cache_file_path}
\title{File path for caching data sets}
\usage{
create_py_dataset_cache_file_path(file_path)
}
\arguments{
\item{file_path}{\code{string} file path without file extension.}
}
\value{
Returns a file path as \code{string}.
}
\description{
Function creates a valid file path for the argument \code{cache_file_name} of classes
\code{"datasets.arrow_dataset.Dataset"} from the python library 'datasets'. The aim of the
function is to ensure compatibility between different versions of 'datasets'.
}
\seealso{
Other Utils Python Data Management Developers: 
\code{\link{class_vector_to_py_dataset}()},
\code{\link{data.frame_to_py_dataset}()},
\code{\link{extract_column_from_py_dataset}()},
\code{\link{get_batches_index}()},
\code{\link{prepare_r_array_for_dataset}()},
\code{\link{py_dataset_to_embeddings}()},
\code{\link{reduce_to_unique}()},
\code{\link{tensor_list_to_numpy}()},
\code{\link{tensor_to_numpy}()}
}
\concept{Utils Python Data Management Developers}
