% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saving_and_loading.R
\name{create_config_state}
\alias{create_config_state}
\title{Create config for R interfaces}
\usage{
create_config_state(object)
}
\arguments{
\item{object}{Object of class \code{"TEClassifierRegular"}, \code{"TEClassifierProtoNet"},
\code{"TEFeatureExtractor"}, \code{"TextEmbeddingModel"}, \code{"LargeDataSetForTextEmbeddings"},
\code{"LargeDataSetForText"}, \code{"EmbeddedText"}.}
}
\value{
Returns a \code{list} that contains the class of the object, the public, and
private fields.
}
\description{
Function creates a config that can be saved to disk. It is used during loading
an object from disk in order to set the correct configuration.
}
\concept{Utils}
\keyword{internal}
