% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python_data_management.R
\name{reduce_to_unique}
\alias{reduce_to_unique}
\title{Reduce to unique cases}
\usage{
reduce_to_unique(dataset_to_reduce, column_name)
}
\arguments{
\item{dataset_to_reduce}{Object of class \code{datasets.arrow_dataset.Dataset}.}

\item{column_name}{\code{string} Name of the column whose values should be unique.}
}
\value{
Returns a data set of class \code{datasets.arrow_dataset.Dataset} where the duplicates are removed according to
the given column.
}
\description{
Function creates an arrow data set that contains only unique cases. That is, duplicates are removed.
}
\seealso{
Other Utils Python Data Management Developers: 
\code{\link{class_vector_to_py_dataset}()},
\code{\link{create_py_dataset_cache_file_path}()},
\code{\link{data.frame_to_py_dataset}()},
\code{\link{extract_column_from_py_dataset}()},
\code{\link{get_batches_index}()},
\code{\link{prepare_r_array_for_dataset}()},
\code{\link{py_dataset_to_embeddings}()},
\code{\link{tensor_list_to_numpy}()},
\code{\link{tensor_to_numpy}()}
}
\concept{Utils Python Data Management Developers}
