% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_Object_Management.R
\name{get_depr_obj_names}
\alias{get_depr_obj_names}
\title{Get names of deprecated objects}
\usage{
get_depr_obj_names()
}
\value{
Returns a \code{vector} containing the names.
}
\description{
Function returns the names of all objects that are deprecated.
}
\seealso{
Other Parameter Dictionary: 
\code{\link{BaseModelsIndex}},
\code{\link{DataSetsIndex}},
\code{\link{TokenizerIndex}},
\code{\link{doc_formula}()},
\code{\link{get_TEClassifiers_class_names}()},
\code{\link{get_called_args}()},
\code{\link{get_magnitude_values}()},
\code{\link{get_param_def}()},
\code{\link{get_param_dict}()},
\code{\link{get_param_doc_desc}()}
}
\concept{Parameter Dictionary}
