% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_log.R
\name{reset_log}
\alias{reset_log}
\title{Function that resets a log file.}
\usage{
reset_log(log_path)
}
\arguments{
\item{log_path}{\code{string} Path to the log file.}
}
\value{
Function does nothing return. It is used to write an "empty" log file.
}
\description{
This function writes a log file with default values. The file can be read with
\link{read_log}.
}
\seealso{
Other Utils Log Developers: 
\code{\link{cat_message}()},
\code{\link{clean_pytorch_log_transformers}()},
\code{\link{output_message}()},
\code{\link{print_message}()},
\code{\link{read_log}()},
\code{\link{read_loss_log}()},
\code{\link{reset_loss_log}()},
\code{\link{write_log}()}
}
\concept{Utils Log Developers}
