% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_log.R
\name{read_log}
\alias{read_log}
\title{Function for reading a log file in R}
\usage{
read_log(file_path)
}
\arguments{
\item{file_path}{\code{string} Path to the log file.}
}
\value{
Returns a matrix containing the log file.
}
\description{
This function reads a log file at the given location. The log file should be
created with \link{write_log}.
}
\seealso{
Other Utils Log Developers: 
\code{\link{cat_message}()},
\code{\link{clean_pytorch_log_transformers}()},
\code{\link{output_message}()},
\code{\link{print_message}()},
\code{\link{read_loss_log}()},
\code{\link{reset_log}()},
\code{\link{reset_loss_log}()},
\code{\link{write_log}()}
}
\concept{Utils Log Developers}
