% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{imdb_movie_reviews}
\alias{imdb_movie_reviews}
\title{Standford Movie Review Dataset}
\format{
data.frame
}
\usage{
imdb_movie_reviews
}
\description{
A \link{data.frame} consisting of a subset of 100 negative and 200 positive movie reviews from the
dataset provided by Maas et al. (2011). The \link{data.frame} consists of three columns. The first column 'text'
stores the movie review. The second stores the labels (0 = negative, 1 = positive). The last column stores the id.
The purpose of the data is for illustration in vignettes and for tests.
}
\references{
Maas, A. L., Daly, R. E., Pham, P. T., Huang, D., Ng, A. Y., & Potts, C. (2011). Learning Word Vectors
for Sentiment Analysis. In D. Lin, Y. Matsumoto, & R. Mihalcea (Eds.), Proceedings of the 49th Annual Meeting of
the Association for Computational Linguistics: Human Language Technologies (pp. 142–150). Association for
Computational Linguistics. <doi: 10.5555/2002472.2002491>
}
\concept{Data Sets}
\keyword{internal}
