% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_Object_Management.R
\docType{data}
\name{TokenizerIndex}
\alias{TokenizerIndex}
\title{List of all available Tokenizers}
\format{
An object of class \code{list} of length 2.
}
\usage{
TokenizerIndex
}
\description{
Named \code{list} containing all tokenizers as a string.
}
\seealso{
Other Parameter Dictionary: 
\code{\link{BaseModelsIndex}},
\code{\link{DataSetsIndex}},
\code{\link{doc_formula}()},
\code{\link{get_TEClassifiers_class_names}()},
\code{\link{get_called_args}()},
\code{\link{get_depr_obj_names}()},
\code{\link{get_magnitude_values}()},
\code{\link{get_param_def}()},
\code{\link{get_param_dict}()},
\code{\link{get_param_doc_desc}()}
}
\concept{Parameter Dictionary}
\keyword{datasets}
