% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_sustainability.R
\name{summarize_tracked_sustainability}
\alias{summarize_tracked_sustainability}
\title{Summarizing tracked sustainability data}
\usage{
summarize_tracked_sustainability(sustainability_tracker, task = NA)
}
\arguments{
\item{sustainability_tracker}{Object of class \code{codecarbon.emissions_tracker.OfflineEmissionsTracker} of the python
library codecarbon.}
}
\value{
Returns a \code{list} which contains the tracked sustainability data.
}
\description{
Function for summarizing the tracked sustainability data with a tracker of the python library
'codecarbon'.
}
\seealso{
Other Utils Sustainability Developers: 
\code{\link{get_alpha_3_codes}()}
}
\concept{Utils Sustainability Developers}
\keyword{internal}
