% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_ParameterDefinitions.R
\name{get_param_def}
\alias{get_param_def}
\title{Definition of an argument}
\usage{
get_param_def(param_name)
}
\arguments{
\item{param_name}{\code{string} Name of the parameter to request its definition.}
}
\value{
Returns a \code{list} with the definition of the argument. See \link{get_param_dict}
for more details.
}
\description{
Function returns the definition of an argument. Please note that
only definitions of arguments can be requested which are used for transformers or
classifier models.
}
\seealso{
Other Parameter Dictionary: 
\code{\link{BaseModelsIndex}},
\code{\link{DataSetsIndex}},
\code{\link{TokenizerIndex}},
\code{\link{doc_formula}()},
\code{\link{get_TEClassifiers_class_names}()},
\code{\link{get_called_args}()},
\code{\link{get_depr_obj_names}()},
\code{\link{get_magnitude_values}()},
\code{\link{get_param_dict}()},
\code{\link{get_param_doc_desc}()}
}
\concept{Parameter Dictionary}
