% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{generate_id}
\alias{generate_id}
\title{Generate ID suffix for objects}
\usage{
generate_id(length = 16L)
}
\arguments{
\item{length}{\code{int} determining the length of the id suffix.}
}
\value{
Returns a \code{string} of the requested length.
}
\description{
Function for generating an ID suffix for objects of class \link{TextEmbeddingModel},
\link{TEClassifierRegular}, and \link{TEClassifierProtoNet}.
}
\seealso{
Other Utils Developers: 
\code{\link{auto_n_cores}()},
\code{\link{create_object}()},
\code{\link{create_synthetic_units_from_matrix}()},
\code{\link{get_n_chunks}()},
\code{\link{get_synthetic_cases_from_matrix}()},
\code{\link{get_time_stamp}()},
\code{\link{matrix_to_array_c}()},
\code{\link{tensor_to_matrix_c}()},
\code{\link{to_categorical_c}()}
}
\concept{Utils Developers}
