% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_sustainability.R
\name{get_alpha_3_codes}
\alias{get_alpha_3_codes}
\title{Country Alpha 3 Codes}
\usage{
get_alpha_3_codes()
}
\value{
Returns a \code{vector} containing the alpha-3 codes for most countries.
}
\description{
Function for requesting a \code{vector} containing the alpha-3 codes for most countries.
}
\seealso{
Other Utils Sustainability Developers: 
\code{\link{summarize_tracked_sustainability}()}
}
\concept{Utils Sustainability Developers}
