% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Studio_Tabs_Reliability.R
\name{Reliability_UI}
\alias{Reliability_UI}
\title{Graphical user interface for displaying the reliability of classifiers.}
\usage{
Reliability_UI(id)
}
\arguments{
\item{id}{\code{string} determining the id for the namespace.}
}
\value{
This function does nothing return. It is used to build a page for a shiny app.
}
\description{
Functions generates the tab within a page for displaying infomration on the reliability of classifiers.
}
\seealso{
Other studio_gui_page_classifier_reliability: 
\code{\link{Reliability_Server}()}
}
\concept{studio_gui_page_classifier_reliability}
\keyword{internal}
