% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_ParameterDefinitions.R
\name{doc_formula}
\alias{doc_formula}
\title{Create rd formula}
\usage{
doc_formula(string_formula)
}
\arguments{
\item{string_formula}{\code{string} Formula expression without specific format.}
}
\value{
Returns a \code{string} which can be used in rd files to display a formula for
different output types.
}
\description{
Function returns the syntax for displaying a formula depending on
the output format.
}
\seealso{
Other Parameter Dictionary: 
\code{\link{BaseModelsIndex}},
\code{\link{DataSetsIndex}},
\code{\link{TokenizerIndex}},
\code{\link{get_TEClassifiers_class_names}()},
\code{\link{get_called_args}()},
\code{\link{get_depr_obj_names}()},
\code{\link{get_magnitude_values}()},
\code{\link{get_param_def}()},
\code{\link{get_param_dict}()},
\code{\link{get_param_doc_desc}()}
}
\concept{Parameter Dictionary}
\keyword{internal}
