% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_check.R
\name{check_all_args}
\alias{check_all_args}
\title{Check arguments automatically}
\usage{
check_all_args(args)
}
\arguments{
\item{args}{Named \code{list} containing the arguments and their values.}
}
\value{
Function does nothing return. It raises an error the arguments are not valid.
}
\description{
This function performs checks for every provided argument. It can
only check arguments that are defined in the central parameter dictionary. See
\link{get_param_dict} for more details.
}
\seealso{
Other Utils Checks Developers: 
\code{\link{check_class_and_type}()}
}
\concept{Utils Checks Developers}
