% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_and_config.R
\name{get_recommended_py_versions}
\alias{get_recommended_py_versions}
\title{Recommended version of python packages}
\usage{
get_recommended_py_versions(package_name = NULL)
}
\arguments{
\item{package_name}{\code{string} Name of the package the recommended version should be returned. If
set to \code{NULL} a table with all core packages and their supported version is returned.}
}
\value{
Returns a \code{data.frame} with the packages in the columns and the minimum,
maximum, and recommended version in the rows. If a concrete name is passed returns a
string with leading '<='.
}
\description{
Returns the minimum and maximum versions of the core
python packages used in \emph{aifeducation}. It is recommended to use packages of
these version. Packages of other versions can result in errors or unexpected results.
}
\seealso{
Other Installation and Configuration: 
\code{\link{check_aif_py_modules}()},
\code{\link{install_aifeducation}()},
\code{\link{install_aifeducation_studio}()},
\code{\link{install_py_modules}()},
\code{\link{prepare_session}()},
\code{\link{set_transformers_logger}()},
\code{\link{update_aifeducation}()}
}
\concept{Installation and Configuration}
