% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_and_config.R
\name{set_transformers_logger}
\alias{set_transformers_logger}
\title{Sets the level for logging information of the 'transformers' library}
\usage{
set_transformers_logger(level = "ERROR")
}
\arguments{
\item{level}{\code{string} Minimal level that should be printed to console. Four levels are available: INFO, WARNING,
ERROR and DEBUG}
}
\value{
This function does not return anything. It is used for its side effects.
}
\description{
This function changes the level for logging information of the 'transformers' library. It influences the
output printed to console for creating and training transformer models as well as \link{TextEmbeddingModel}s.
}
\seealso{
Other Installation and Configuration: 
\code{\link{check_aif_py_modules}()},
\code{\link{get_recommended_py_versions}()},
\code{\link{install_aifeducation}()},
\code{\link{install_aifeducation_studio}()},
\code{\link{install_py_modules}()},
\code{\link{prepare_session}()},
\code{\link{update_aifeducation}()}
}
\concept{Installation and Configuration}
