% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_log.R
\name{print_message}
\alias{print_message}
\title{Print message (\code{message()})}
\usage{
print_message(msg, trace)
}
\arguments{
\item{msg}{\code{string} Message that should be printed.}

\item{trace}{\code{bool} Silent printing (\code{FALSE}) or not (\code{TRUE}).}
}
\value{
This function returns nothing.
}
\description{
Prints a message \code{msg} if \code{trace} parameter is \code{TRUE} with current date with \code{message()} function.
}
\seealso{
Other Utils Log Developers: 
\code{\link{cat_message}()},
\code{\link{clean_pytorch_log_transformers}()},
\code{\link{output_message}()},
\code{\link{read_log}()},
\code{\link{read_loss_log}()},
\code{\link{reset_log}()},
\code{\link{reset_loss_log}()},
\code{\link{write_log}()}
}
\concept{Utils Log Developers}
