% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saving_and_loading.R
\name{save_to_disk}
\alias{save_to_disk}
\title{Saving objects created with 'aifeducation'}
\usage{
save_to_disk(object, dir_path, folder_name)
}
\arguments{
\item{object}{Object of class \link{TEClassifierRegular}, \link{TEClassifierProtoNet},  \link{TEFeatureExtractor},
\link{TextEmbeddingModel}, \link{LargeDataSetForTextEmbeddings}, \link{LargeDataSetForText} or \link{EmbeddedText} which should be
saved.}

\item{dir_path}{\code{string} Path to the directory where the should model is stored.}

\item{folder_name}{\code{string} Name of the folder where the files should be stored.}
}
\value{
Function does not return a value. It saves the model to disk.

No return value, called for side effects.
}
\description{
Function for saving objects created with 'aifeducation'.
}
\seealso{
Other Saving and Loading: 
\code{\link{load_from_disk}()}
}
\concept{Saving and Loading}
