% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_test_that.R
\name{get_test_data_for_classifiers}
\alias{get_test_data_for_classifiers}
\title{Get test data}
\usage{
get_test_data_for_classifiers(class_range = c(2L, 3L), path_test_embeddings)
}
\arguments{
\item{class_range}{\code{vector} containing the number of classes.}

\item{path_test_embeddings}{\code{string} Path to the location where the test data is stored.}
}
\value{
Returns a \code{list} with test data.
}
\description{
Function returns example data for testing the package
}
\seealso{
Other Utils TestThat Developers: 
\code{\link{check_adjust_n_samples_on_CI}()},
\code{\link{generate_args_for_tests}()},
\code{\link{generate_embeddings}()},
\code{\link{generate_tensors}()},
\code{\link{get_current_args_for_print}()},
\code{\link{get_fixed_test_tensor}()},
\code{\link{monitor_test_time_on_CI}()},
\code{\link{random_bool_on_CI}()}
}
\concept{Utils TestThat Developers}
