% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_test_that.R
\name{monitor_test_time_on_CI}
\alias{monitor_test_time_on_CI}
\title{Print duration of a test on CI}
\usage{
monitor_test_time_on_CI(start_time, test_name)
}
\arguments{
\item{start_time}{\code{POSIXct} Start time of the test.}

\item{test_name}{\code{string} Name of the test to print.}
}
\value{
Returns nothing.
}
\description{
Function prints the duration of a test to console if the test is
running on CI. If not no output appears in console.
}
\seealso{
Other Utils TestThat Developers: 
\code{\link{check_adjust_n_samples_on_CI}()},
\code{\link{generate_args_for_tests}()},
\code{\link{generate_embeddings}()},
\code{\link{generate_tensors}()},
\code{\link{get_current_args_for_print}()},
\code{\link{get_fixed_test_tensor}()},
\code{\link{get_test_data_for_classifiers}()},
\code{\link{random_bool_on_CI}()}
}
\concept{Utils TestThat Developers}
