% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_DocumentationDesc.R
\name{build_documentation_for_model}
\alias{build_documentation_for_model}
\title{Generate documentation for a classifier class}
\usage{
build_documentation_for_model(
  model_name,
  cls_type = NULL,
  core_type = NULL,
  input_type = "text_embeddings"
)
}
\arguments{
\item{model_name}{\code{string} Name of the model.}

\item{cls_type}{\code{string} Type of classification}

\item{core_type}{\code{string} Name of the core type.}

\item{input_type}{\code{bool} Name of the input type necessary for training and predicting.}
}
\value{
Returns a \code{string} containing the description written in rmarkdown.
}
\description{
Function for generating the documentation of a model.
}
\note{
Function is designed to be used with roxygen2 in the regular documentation.
}
\seealso{
Other Utils Documentation: 
\code{\link{build_layer_stack_documentation_for_vignette}()},
\code{\link{get_desc_for_core_model_architecture}()},
\code{\link{get_dict_cls_type}()},
\code{\link{get_dict_core_models}()},
\code{\link{get_dict_input_types}()},
\code{\link{get_layer_dict}()},
\code{\link{get_layer_documentation}()},
\code{\link{get_parameter_documentation}()}
}
\concept{Utils Documentation}
