% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_log.R
\name{reset_loss_log}
\alias{reset_loss_log}
\title{Reset log for loss information}
\usage{
reset_loss_log(log_path, epochs)
}
\arguments{
\item{log_path}{\code{string} Path to the log file.}

\item{epochs}{\code{int} Number of epochs for the complete training process.}
}
\value{
Function does nothing return. It writes a log file at the given location.
The file is a .csv file that contains three rows. The first row takes the
value for the training, the second for the validation, and the third row for the
test data. The columns represent epochs.
}
\description{
This function writes an empty log file for loss information.
}
\seealso{
Other Utils Log Developers: 
\code{\link{cat_message}()},
\code{\link{clean_pytorch_log_transformers}()},
\code{\link{output_message}()},
\code{\link{print_message}()},
\code{\link{read_log}()},
\code{\link{read_loss_log}()},
\code{\link{reset_log}()},
\code{\link{write_log}()}
}
\concept{Utils Log Developers}
