% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_test_that.R
\name{get_current_args_for_print}
\alias{get_current_args_for_print}
\title{Print arguments}
\usage{
get_current_args_for_print(arg_list)
}
\arguments{
\item{arg_list}{Named \code{list} of arguments. The list should be generated with \link{generate_args_for_tests}.}
}
\value{
Function does nothing return.
}
\description{
Functions prints the used arguments. The aim of this function is
to print the arguments to the console that resulted in a failed test.
}
\seealso{
Other Utils TestThat Developers: 
\code{\link{check_adjust_n_samples_on_CI}()},
\code{\link{generate_args_for_tests}()},
\code{\link{generate_embeddings}()},
\code{\link{generate_tensors}()},
\code{\link{get_fixed_test_tensor}()},
\code{\link{get_test_data_for_classifiers}()},
\code{\link{monitor_test_time_on_CI}()},
\code{\link{random_bool_on_CI}()}
}
\concept{Utils TestThat Developers}
