% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/performance_measures.R
\name{calc_standard_classification_measures}
\alias{calc_standard_classification_measures}
\title{Calculate recall, precision, and f1-scores}
\usage{
calc_standard_classification_measures(true_values, predicted_values)
}
\arguments{
\item{true_values}{\code{factor} containing the true labels/categories.}

\item{predicted_values}{\code{factor} containing the predicted labels/categories.}
}
\value{
Returns a matrix which contains the cases categories in the rows and the measures (precision, recall, f1) in
the columns.
}
\description{
Function for calculating recall, precision, and f1-scores.
}
\seealso{
Other performance measures: 
\code{\link{cohens_kappa}()},
\code{\link{fleiss_kappa}()},
\code{\link{get_coder_metrics}()},
\code{\link{gwet_ac}()},
\code{\link{kendalls_w}()},
\code{\link{kripp_alpha}()}
}
\concept{performance measures}
