% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_Studio.R
\name{add_missing_args}
\alias{add_missing_args}
\title{Add missing arguments to a list of arguments}
\usage{
add_missing_args(args, path_args, meta_args)
}
\arguments{
\item{args}{Named \code{list} List for arguments for the method of a specific class.}

\item{path_args}{Named \code{list} List of paths where the objects are stored on disk.}

\item{meta_args}{Named \code{list} List containing arguments that are necessary in order to
add the missing objects correctly.}
}
\value{
Returns a named \code{list} of all arguments that a method of a specific class
requires.
}
\description{
This function is designed for taking the output of
\code{summarize_args_for_long_task} as input. It adds the missing arguments.
In general these are arguments that rely on objects of class R6 which can not
be exported to a new R session.
}
\seealso{
Other Utils Studio Developers: 
\code{\link{create_data_embeddings_description}()},
\code{\link{long_load_target_data}()},
\code{\link{summarize_args_for_long_task}()}
}
\concept{Utils Studio Developers}
