% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Studio_Tabs_Reliability.R
\name{Reliability_Server}
\alias{Reliability_Server}
\title{Server function for: graphical user interface for displaying the reliability of classifiers.}
\usage{
Reliability_Server(id, model)
}
\arguments{
\item{id}{\code{string} determining the id for the namespace.}

\item{model}{Model used for inference.}
}
\value{
This function does nothing return. It is used to create the functionality of a page for a shiny app.
}
\description{
Functions generates the functionality of a page on the server.
}
\seealso{
Other studio_gui_page_classifier_reliability: 
\code{\link{Reliability_UI}()}
}
\concept{studio_gui_page_classifier_reliability}
\keyword{internal}
