% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python_data_management.R
\name{tensor_to_numpy}
\alias{tensor_to_numpy}
\title{Tensor_to_numpy}
\usage{
tensor_to_numpy(object)
}
\arguments{
\item{object}{Object of any class.}
}
\value{
In the case the object is of class \code{torch.Tensor} it returns a numpy error.
If the tensor requires a gradient and/or is on gpu it is detached and converted.
If the object is not of class \code{torch.Tensor} the original object is returned.
}
\description{
Function converts a tensor into a numpy array in order to allow further operations in \emph{R}.
}
\seealso{
Other Utils Python Data Management Developers: 
\code{\link{class_vector_to_py_dataset}()},
\code{\link{create_py_dataset_cache_file_path}()},
\code{\link{data.frame_to_py_dataset}()},
\code{\link{extract_column_from_py_dataset}()},
\code{\link{get_batches_index}()},
\code{\link{prepare_r_array_for_dataset}()},
\code{\link{py_dataset_to_embeddings}()},
\code{\link{reduce_to_unique}()},
\code{\link{tensor_list_to_numpy}()}
}
\concept{Utils Python Data Management Developers}
