% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{knnor_is_same_class}
\alias{knnor_is_same_class}
\title{Validate a new point}
\usage{
knnor_is_same_class(new_point, dataset, labels, k)
}
\arguments{
\item{new_point}{\verb{1-D array (vector)} new data point to be validated before adding (with \code{times*features} elements)}

\item{dataset}{\verb{2-D array (matrix)} current embeddings (with size \code{batch} x \code{times*features})}

\item{labels}{\verb{1-D array (vector)} of integers with \code{batch} elements}

\item{k}{\verb{unsigned integer} number of nearest neighbors}
}
\value{
Returns \code{TRUE} if a new point can be added, otherwise - \code{FALSE}
}
\description{
Function written in C++ for validating a new point (KNNOR-Validation)
}
\concept{oversampling_approaches Developers}
