% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_log.R
\name{clean_pytorch_log_transformers}
\alias{clean_pytorch_log_transformers}
\title{Clean pytorch log of transformers}
\usage{
clean_pytorch_log_transformers(log)
}
\arguments{
\item{log}{\code{data.frame} containing the log.}
}
\value{
Returns a \code{data.frame} containing epochs, loss, and val_loss.
}
\description{
Function for preparing and cleaning the log created by an object of class Trainer from the python
library 'transformer's.
}
\seealso{
Other Utils Log Developers: 
\code{\link{cat_message}()},
\code{\link{output_message}()},
\code{\link{print_message}()},
\code{\link{read_log}()},
\code{\link{read_loss_log}()},
\code{\link{reset_log}()},
\code{\link{reset_loss_log}()},
\code{\link{write_log}()}
}
\concept{Utils Log Developers}
