% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_DocumentationDesc.R
\name{build_layer_stack_documentation_for_vignette}
\alias{build_layer_stack_documentation_for_vignette}
\title{Generate documentation of all layers for an vignette or article}
\usage{
build_layer_stack_documentation_for_vignette()
}
\value{
Returns a \code{string} containing the description written in rmarkdown.
}
\description{
Function for generating the whole documentation for an article
used on the package's home page.
}
\note{
Function is designed to be used with inline r code in rmarkdown vignettes/articles.
}
\seealso{
Other Utils Documentation: 
\code{\link{build_documentation_for_model}()},
\code{\link{get_desc_for_core_model_architecture}()},
\code{\link{get_dict_cls_type}()},
\code{\link{get_dict_core_models}()},
\code{\link{get_dict_input_types}()},
\code{\link{get_layer_dict}()},
\code{\link{get_layer_documentation}()},
\code{\link{get_parameter_documentation}()}
}
\concept{Utils Documentation}
