% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_test_that.R
\name{get_fixed_test_tensor}
\alias{get_fixed_test_tensor}
\title{Generate static test tensor}
\usage{
get_fixed_test_tensor(pad_value)
}
\arguments{
\item{pad_value}{\code{int} Value used to indicate padding.}
}
\value{
Returns an object of class \code{Tensor} which is always the same except padding.
Shape (5,3,7).
}
\description{
Function generates a static test tensor which is always the same.
}
\seealso{
Other Utils TestThat Developers: 
\code{\link{check_adjust_n_samples_on_CI}()},
\code{\link{generate_args_for_tests}()},
\code{\link{generate_embeddings}()},
\code{\link{generate_tensors}()},
\code{\link{get_current_args_for_print}()},
\code{\link{get_test_data_for_classifiers}()},
\code{\link{monitor_test_time_on_CI}()},
\code{\link{random_bool_on_CI}()}
}
\concept{Utils TestThat Developers}
