% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python.R
\name{get_py_package_version}
\alias{get_py_package_version}
\title{Get versions of a specific python package}
\usage{
get_py_package_version(package_name)
}
\arguments{
\item{package_name}{\code{string} Name of the package.}
}
\value{
Returns the version as \code{string} or \code{NA} if the package does not exist
or no version is available.
}
\description{
Function for requesting the version of a specific python package.
}
\seealso{
Other Utils Python Developers: 
\code{\link{get_py_package_versions}()},
\code{\link{load_all_py_scripts}()},
\code{\link{load_py_scripts}()},
\code{\link{run_py_file}()}
}
\concept{Utils Python Developers}
