% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_test_that.R
\name{check_adjust_n_samples_on_CI}
\alias{check_adjust_n_samples_on_CI}
\title{Set sample size for argument combinations}
\usage{
check_adjust_n_samples_on_CI(n_samples_requested, n_CI = 50L)
}
\arguments{
\item{n_samples_requested}{\code{int} Number of samples if the test do not run on continuous integration.}

\item{n_CI}{\code{int} Number of samples if the test run on continuous integration.}
}
\value{
Returns an \code{int} depending on the test environment.
}
\description{
Depending on the test environment, the function adjusts the number of samples.
For continuous integration, it is limited to a random sample of combinations.
The same applies if CUDA is unavailable.
}
\seealso{
Other Utils TestThat Developers: 
\code{\link{generate_args_for_tests}()},
\code{\link{generate_embeddings}()},
\code{\link{generate_tensors}()},
\code{\link{get_current_args_for_print}()},
\code{\link{get_fixed_test_tensor}()},
\code{\link{get_test_data_for_classifiers}()},
\code{\link{monitor_test_time_on_CI}()},
\code{\link{random_bool_on_CI}()}
}
\concept{Utils TestThat Developers}
