% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saving_and_loading.R
\name{load_from_disk}
\alias{load_from_disk}
\title{Loading objects created with 'aifeducation'}
\usage{
load_from_disk(dir_path)
}
\arguments{
\item{dir_path}{\code{string} Path to the directory where the model is stored.}
}
\value{
Returns an object of class \link{TEClassifierRegular}, \link{TEClassifierProtoNet},  \link{TEFeatureExtractor},
\link{TextEmbeddingModel}, \link{LargeDataSetForTextEmbeddings}, \link{LargeDataSetForText} or \link{EmbeddedText}.
}
\description{
Function for loading objects created with 'aifeducation'.
}
\seealso{
Other Saving and Loading: 
\code{\link{save_to_disk}()}
}
\concept{Saving and Loading}
