% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_test_that.R
\name{random_bool_on_CI}
\alias{random_bool_on_CI}
\title{Random bool on Continuous Integration}
\usage{
random_bool_on_CI()
}
\value{
Returns a \code{bool}.
}
\description{
Function returns randomly \code{TRUE} or \code{FALSE} if on CI. It returns \code{FALSE} if it is
not on CI.
}
\seealso{
Other Utils TestThat Developers: 
\code{\link{check_adjust_n_samples_on_CI}()},
\code{\link{generate_args_for_tests}()},
\code{\link{generate_embeddings}()},
\code{\link{generate_tensors}()},
\code{\link{get_current_args_for_print}()},
\code{\link{get_fixed_test_tensor}()},
\code{\link{get_test_data_for_classifiers}()},
\code{\link{monitor_test_time_on_CI}()}
}
\concept{Utils TestThat Developers}
