% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_memory_cache.R
\name{inspect_tmp_dir}
\alias{inspect_tmp_dir}
\title{Inspect Temporary directory}
\usage{
inspect_tmp_dir()
}
\value{
Returns a \code{list} containing a \code{vector} with the paths of all files in
the temporary directory and the cumulative file size in bytes.
}
\description{
Function reporting the number of files and the cumulative size
of the files in temporary directory.
}
\concept{Memory Cache}
