% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python_data_management.R
\name{class_vector_to_py_dataset}
\alias{class_vector_to_py_dataset}
\title{Convert class vector to arrow data set}
\usage{
class_vector_to_py_dataset(vector)
}
\arguments{
\item{vector}{\code{vector} of class indices.}
}
\value{
Returns a data set of class \code{datasets.arrow_dataset.Dataset} containing the class indices.
}
\description{
Function converts a vector of class indices into an arrow data set.
}
\seealso{
Other Utils Python Data Management Developers: 
\code{\link{create_py_dataset_cache_file_path}()},
\code{\link{data.frame_to_py_dataset}()},
\code{\link{extract_column_from_py_dataset}()},
\code{\link{get_batches_index}()},
\code{\link{prepare_r_array_for_dataset}()},
\code{\link{py_dataset_to_embeddings}()},
\code{\link{reduce_to_unique}()},
\code{\link{tensor_list_to_numpy}()},
\code{\link{tensor_to_numpy}()}
}
\concept{Utils Python Data Management Developers}
