% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python.R
\name{run_py_file}
\alias{run_py_file}
\title{Run python file}
\usage{
run_py_file(py_file_name)
}
\arguments{
\item{py_file_name}{\code{string} Name of a python file to run. The file must be in the \code{python} folder of \code{aifeducation}
package.}
}
\value{
This function returns nothing.
}
\description{
Used to run python files with \code{reticulate::py_run_file()} from folder \code{python}.
}
\seealso{
Other Utils Python Developers: 
\code{\link{get_py_package_version}()},
\code{\link{get_py_package_versions}()},
\code{\link{load_all_py_scripts}()},
\code{\link{load_py_scripts}()}
}
\concept{Utils Python Developers}
