% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_general.R
\name{get_time_stamp}
\alias{get_time_stamp}
\title{Time stamp}
\usage{
get_time_stamp()
}
\value{
Returns a \code{string} with date and time in format "\%y-\%m-\%d \%H:\%M:\%S".
}
\description{
Function returns the time on the machine at the moment of calling.
}
\seealso{
Other Utils Developers: 
\code{\link{auto_n_cores}()},
\code{\link{create_object}()},
\code{\link{create_synthetic_units_from_matrix}()},
\code{\link{generate_id}()},
\code{\link{get_n_chunks}()},
\code{\link{get_synthetic_cases_from_matrix}()},
\code{\link{matrix_to_array_c}()},
\code{\link{tensor_to_matrix_c}()},
\code{\link{to_categorical_c}()}
}
\concept{Utils Developers}
