% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/def_Object_Management.R
\name{get_TEClassifiers_class_names}
\alias{get_TEClassifiers_class_names}
\title{Get names of classifiers}
\usage{
get_TEClassifiers_class_names(super_class = NULL)
}
\arguments{
\item{super_class}{\code{string} Name of the super class the classifiers should
be a child of. To request the names of all classifiers set this argument to \code{NULL}.}
}
\value{
Returns a vector containing the names of the classifiers.
}
\description{
Function returns the names of all classifiers which
are child classes of a specific super class.
}
\seealso{
Other Parameter Dictionary: 
\code{\link{BaseModelsIndex}},
\code{\link{DataSetsIndex}},
\code{\link{TokenizerIndex}},
\code{\link{doc_formula}()},
\code{\link{get_called_args}()},
\code{\link{get_depr_obj_names}()},
\code{\link{get_magnitude_values}()},
\code{\link{get_param_def}()},
\code{\link{get_param_dict}()},
\code{\link{get_param_doc_desc}()}
}
\concept{Parameter Dictionary}
