% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/install_and_config.R
\name{install_aifeducation_studio}
\alias{install_aifeducation_studio}
\title{Install 'AI for Education - Studio' on a machine}
\usage{
install_aifeducation_studio()
}
\value{
Function does nothing return. It installs/updates R packages.
}
\description{
Function installs/updates all relevant R packages necessary
to run the shiny app ''AI for Education - Studio'.
}
\seealso{
Other Installation and Configuration: 
\code{\link{check_aif_py_modules}()},
\code{\link{get_recommended_py_versions}()},
\code{\link{install_aifeducation}()},
\code{\link{install_py_modules}()},
\code{\link{prepare_session}()},
\code{\link{set_transformers_logger}()},
\code{\link{update_aifeducation}()}
}
\concept{Installation and Configuration}
