% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python_data_management.R
\name{get_batches_index}
\alias{get_batches_index}
\title{Assign cases to batches}
\usage{
get_batches_index(number_rows, batch_size, zero_based = FALSE)
}
\arguments{
\item{number_rows}{\code{int} representing the number of cases or rows of a matrix or array.}

\item{batch_size}{\code{int} size of a batch.}

\item{zero_based}{\code{bool} If \code{TRUE} the indices of the cases within each batch are zero based. One based if \code{FALSE}.}
}
\value{
Returns a \code{list} of batches. Each entry in the list contains a \code{vector} of \code{int} representing the cases
belonging to that batch.
}
\description{
Function groups cases into batches.
}
\seealso{
Other Utils Python Data Management Developers: 
\code{\link{class_vector_to_py_dataset}()},
\code{\link{create_py_dataset_cache_file_path}()},
\code{\link{data.frame_to_py_dataset}()},
\code{\link{extract_column_from_py_dataset}()},
\code{\link{prepare_r_array_for_dataset}()},
\code{\link{py_dataset_to_embeddings}()},
\code{\link{reduce_to_unique}()},
\code{\link{tensor_list_to_numpy}()},
\code{\link{tensor_to_numpy}()}
}
\concept{Utils Python Data Management Developers}
