% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python_data_management.R
\name{py_dataset_to_embeddings}
\alias{py_dataset_to_embeddings}
\title{Convert arrow data set to an arrow data set}
\usage{
py_dataset_to_embeddings(py_dataset)
}
\arguments{
\item{py_dataset}{Object of class \code{datasets.arrow_dataset.Dataset}.}
}
\value{
Returns the data set of class \code{datasets.arrow_dataset.Dataset} with only two columns ("id","input"). "id"
stores the name of the cases while "input" stores the embeddings.
}
\description{
Function for converting an arrow data set into a data set that can be used to store and process
embeddings.
}
\seealso{
Other Utils Python Data Management Developers: 
\code{\link{class_vector_to_py_dataset}()},
\code{\link{create_py_dataset_cache_file_path}()},
\code{\link{data.frame_to_py_dataset}()},
\code{\link{extract_column_from_py_dataset}()},
\code{\link{get_batches_index}()},
\code{\link{prepare_r_array_for_dataset}()},
\code{\link{reduce_to_unique}()},
\code{\link{tensor_list_to_numpy}()},
\code{\link{tensor_to_numpy}()}
}
\concept{Utils Python Data Management Developers}
