% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_file_management.R
\name{get_file_extension}
\alias{get_file_extension}
\title{Get file extension}
\usage{
get_file_extension(file_path)
}
\arguments{
\item{file_path}{\code{string} Path to a file.}
}
\value{
Returns the extension of a file as a string.
}
\description{
Function for requesting the file extension
}
\seealso{
Other Utils File Management Developers: 
\code{\link{create_dir}()}
}
\concept{Utils File Management Developers}
