% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_python_data_management.R
\name{tensor_list_to_numpy}
\alias{tensor_list_to_numpy}
\title{Convert list of tensors into numpy arrays}
\usage{
tensor_list_to_numpy(tensor_list)
}
\arguments{
\item{tensor_list}{\code{list} of objects.}
}
\value{
Returns the same \code{list} with the exception that objects of class \code{torch.Tensor} are transformed into numpy arrays.
If the tensor requires a gradient and/or is on gpu it is detached and converted.
If the object in a list is not of this class the original object is returned.
}
\description{
Function converts tensors within a \code{list} into numpy arrays in order to allow further operations in \emph{R}.
}
\seealso{
Other Utils Python Data Management Developers: 
\code{\link{class_vector_to_py_dataset}()},
\code{\link{create_py_dataset_cache_file_path}()},
\code{\link{data.frame_to_py_dataset}()},
\code{\link{extract_column_from_py_dataset}()},
\code{\link{get_batches_index}()},
\code{\link{prepare_r_array_for_dataset}()},
\code{\link{py_dataset_to_embeddings}()},
\code{\link{reduce_to_unique}()},
\code{\link{tensor_to_numpy}()}
}
\concept{Utils Python Data Management Developers}
