% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_file_management.R
\name{create_dir}
\alias{create_dir}
\title{Create directory if not exists}
\usage{
create_dir(dir_path, trace, msg = "Creating Directory", msg_fun = TRUE)
}
\arguments{
\item{dir_path}{\code{string} A new directory path that should be created.}

\item{trace}{\code{bool} Whether a \code{msg} message should be printed.}

\item{msg}{\code{string} A message that should be printed if \code{trace} is \code{TRUE}.}

\item{msg_fun}{\code{func} Function used for printing the message.}
}
\value{
\code{TRUE} or \code{FALSE} depending on whether the shiny app is active.
}
\description{
Check whether the passed \code{dir_path} directory exists. If not, creates a new directory and prints a \code{msg}
message if \code{trace} is \code{TRUE}.
}
\seealso{
Other Utils File Management Developers: 
\code{\link{get_file_extension}()}
}
\concept{Utils File Management Developers}
