% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_DataManager.R
\name{get_synthetic_cases_from_matrix}
\alias{get_synthetic_cases_from_matrix}
\title{Create synthetic cases for balancing training data}
\usage{
get_synthetic_cases_from_matrix(
  matrix_form,
  times,
  features,
  target,
  sequence_length,
  method = "knnor",
  min_k = 1L,
  max_k = 6L,
  pad_value = -100L
)
}
\arguments{
\item{matrix_form}{Named \code{matrix} containing the text embeddings in a matrix form.}

\item{times}{\code{int} for the number of sequences/times.}

\item{features}{\code{int} for the number of features within each sequence.}

\item{target}{Named \code{factor} containing the labels of the corresponding embeddings.}

\item{sequence_length}{\code{int} Length of the text embedding sequences.}

\item{method}{\code{vector} containing strings of the requested methods for generating new cases. Currently
"knnor" from this package is available.}

\item{min_k}{\code{int} The minimal number of nearest neighbors during sampling process.}

\item{max_k}{\code{int} The maximum number of nearest neighbors during sampling process.}

\item{pad_value}{\code{int} Value for indicating padding.}
}
\value{
\code{list} with the following components:
\itemize{
\item \code{syntetic_embeddings}: Named \code{data.frame} containing the text embeddings of the synthetic cases.
\item \code{syntetic_targets}: Named \code{factor} containing the labels of the corresponding synthetic cases.
\item \code{n_syntetic_units}: \code{table} showing the number of synthetic cases for every label/category.
}
}
\description{
This function creates synthetic cases for balancing the training with classifier models.
}
\seealso{
Other Utils Developers: 
\code{\link{auto_n_cores}()},
\code{\link{create_object}()},
\code{\link{create_synthetic_units_from_matrix}()},
\code{\link{generate_id}()},
\code{\link{get_n_chunks}()},
\code{\link{get_time_stamp}()},
\code{\link{matrix_to_array_c}()},
\code{\link{tensor_to_matrix_c}()},
\code{\link{to_categorical_c}()}
}
\concept{Utils Developers}
